import { MyGlobal as app } from './globals';
import { MiddlewareParams, ValidObject } from './types';
import axios from './axios';
import Redis from './redis';

let ShopifyApiMiddleware = {
    bind: (args: MiddlewareParams) => {
        app.logger = args.logger;
        app.FlclError = args.FlclError;
        app.tokenizedCall = args.tokenizedCall || false;
        if(app.tokenizedCall) {
            app.tokenManagerParams = {
                // storeName: args.storeName,
                REDIS_HOST: args.REDIS_HOST,
                REDIS_PORT: args.REDIS_PORT,
                REDIS_DB: args.REDIS_DB,
                REDIS_PASS: args.REDIS_PASS
            }
            app.redisInstance = new Redis();
        }
        app.logger?.info({npm:'fc-shopify-middleware', className: 'ShopifyApiMiddleware', methodName: 'bind', propertyName: 'SHOPIFY_MIDDLEWARE_ARGS_RECEIVED', propertyValue: {tokenizedCall: args.tokenizedCall, REDIS_HOST: args.REDIS_HOST, REDIS_PORT: args.REDIS_PORT, REDIS_DB: args.REDIS_DB, REDIS_PASS: args.REDIS_PASS }});
    },
    get: async (url: string, configs?: ValidObject, options?: ValidObject): Promise<any> => {
        try {
            app.logger?.info({ npm: 'fc-shopify-middleware', className: 'ShopifyApiMiddleware', methodName: 'GET', propertyName: 'REQUEST', propertyValue: { url } });
            let resp = await axios.get(url, configs, options);
            app.logger?.info({npm:'fc-shopify-middleware', className: 'ShopifyApiMiddleware', methodName: 'GET', propertyName: 'RESPONSE_CAME_BACK'});
            if(app.tokenizedCall)
                await updateCreditsLimit(resp, options);
            return resp;
        } catch(e) {
            let retryApiCall = shouldRetry(e, options);
            if(retryApiCall && options) {
                options.retryAttempsMade = options.retryAttempsMade || 0;
                options.retryAttempsMade++;
                if(app.tokenizedCall)
                    await app.redisInstance?.client?.SET(`leaked:count:${options?.storeName}`, "-1");
                app.logger?.info({npm:'fc-shopify-middleware', className: 'ShopifyApiMiddleware', methodName: 'GET', propertyName: 'SHOPIFY_API_RETRY_ON_429', propertyValue: {url}, message: {retryAttempsMade: options.retryAttempsMade, retryOnEx: options?.retryOnEx, msg: 'Had set the credit available as -1 in redis'} });
                return await ShopifyApiMiddleware.get(url, configs, {...options, retryOnEx: true});
            }
            app.logger?.error(app.FlclError?.create({npm:'fc-shopify-middleware', className: 'ShopifyApiMiddleware', methodName: 'GET', propertyName: 'SHOPIFY_API_ERROR', cause: e, message: 'Exception got from Axios', propertyValue: {config: e.config, retryOnEx: options?.retryOnEx, retryAttempsMade: options?.retryAttempsMade}}));
            if(app.tokenizedCall)
                await updateCreditsLimit(e?.response, options);
            throw e;
        }
    },
    post: async (url: string, data?: ValidObject,  configs?: ValidObject, options?: ValidObject): Promise<any> => {
        try {
            let resp = await axios.post(url, data, configs, options);
            if(app.tokenizedCall)
                await updateCreditsLimit(resp, options);
            app.logger?.info({npm:'fc-shopify-middleware', className: 'ShopifyApiMiddleware', methodName: 'post', propertyName: 'RESPONSE_CAME_BACK'});
            return resp;
        } catch(e) {
            let retryApiCall = shouldRetry(e, options);
            if(retryApiCall && options) {
                options.retryAttempsMade = options.retryAttempsMade || 0;
                options.retryAttempsMade++;
                if(app.tokenizedCall)
                    await app.redisInstance?.client?.SET(`leaked:count:${options?.storeName}`, "-1");
                app.logger?.info({npm:'fc-shopify-middleware', className: 'ShopifyApiMiddleware', methodName: 'post', propertyName: 'SHOPIFY_API_RETRY_ON_429', propertyValue: {url}, message: {retryAttempsMade: options.retryAttempsMade, retryOnEx: options?.retryOnEx, msg: 'Had set the credit available as -1 in redis'} });
                return await ShopifyApiMiddleware.post(url, data, configs, {...options, retryOnEx: true});
            }
            app.logger?.error(app.FlclError?.create({npm:'fc-shopify-middleware', className: 'ShopifyApiMiddleware', methodName: 'post', propertyName: 'SHOPIFY_API_ERROR', cause: e, message: 'Exception got from Axios', propertyValue: {config: e.config, retryOnEx: options?.retryOnEx, retryAttempsMade: options?.retryAttempsMade}}));
            if(app.tokenizedCall)
                await updateCreditsLimit(e?.response, options);
            throw e;
        }
    },
    put: async (url: string, data?: ValidObject, configs?: ValidObject, options?: ValidObject): Promise<any> => {
        try {
            let resp = await axios.put(url, data, configs, options);
            if (app.tokenizedCall)
                await updateCreditsLimit(resp, options);
            app.logger?.info({ npm: 'fc-shopify-middleware', className: 'ShopifyApiMiddleware', methodName: 'put', propertyName: 'RESPONSE_CAME_BACK' });
            return resp;
        } catch (e) {
            let retryApiCall = shouldRetry(e, options);
            if (retryApiCall && options) {
                options.retryAttempsMade = options.retryAttempsMade || 0;
                options.retryAttempsMade++;
                if (app.tokenizedCall)
                    await app.redisInstance?.client?.SET(`leaked:count:${options?.storeName}`, "-1");
                app.logger?.info({ npm: 'fc-shopify-middleware', className: 'ShopifyApiMiddleware', methodName: 'put', propertyName: 'SHOPIFY_API_RETRY_ON_429', propertyValue: { url }, message: { retryAttempsMade: options.retryAttempsMade, retryOnEx: options?.retryOnEx, msg: 'Had set the credit available as -1 in redis' } });
                return await ShopifyApiMiddleware.put(url, data, configs, { ...options, retryOnEx: true });
            }
            app.logger?.error(app.FlclError?.create({ npm: 'fc-shopify-middleware', className: 'ShopifyApiMiddleware', methodName: 'put', propertyName: 'SHOPIFY_API_ERROR', cause: e, message: 'Exception got from Axios', propertyValue: { config: e.config, retryOnEx: options?.retryOnEx, retryAttempsMade: options?.retryAttempsMade } }));
            if (app.tokenizedCall)
                await updateCreditsLimit(e?.response, options);
            throw e;
        }
    }
}

let updateCreditsLimit = async (resp: ValidObject, options?: ValidObject) => {
    try {
        if(resp && resp.headers) {
            let xShopifyApiCallLimit = resp.headers["x-shopify-shop-api-call-limit"];
            if(xShopifyApiCallLimit) {
                let splits = xShopifyApiCallLimit.split("/");
                let usedLimit = splits[0];
                let maxLimit = parseInt(splits[1]);
                let remainingLimit = maxLimit - usedLimit;
                app.logger?.info({npm:'fc-shopify-middleware', className: 'ShopifyApiMiddleware', methodName: 'updateCreditsLimit', propertyName: 'SET_CREDITS_AFTER_RESP', propertyValue: remainingLimit});
                await app.redisInstance?.client?.SET(`leaked:count:${options?.storeName}`, remainingLimit.toString());
            }
        } else {
            app.logger?.info({npm:'fc-shopify-middleware', className: 'ShopifyApiMiddleware', methodName: 'updateCreditsLimit', propertyName: 'RESP_HEADERS_NOT_AVL'});
        }
        return resp;
    } catch(e) {
        app.logger?.info({npm:'fc-shopify-middleware', className: 'ShopifyApiMiddleware', methodName: 'updateCreditsLimit', propertyName: 'SHOPIFY_API_RESP_HANDLING_ERROR', cause: e, message: 'Exception got while updating redis after shopify call'});
        return resp;
    }
}

let shouldRetry = (e: any, options?: ValidObject) => {
    let flag = false;
    if(e.response && e.response.status == 429) {
        flag = true;
        if(options?.retryAttempsMade && options?.retryAttempsMade > 2)
            flag = false;
    }
    return flag;
}
export default ShopifyApiMiddleware;